#ifndef _VMS_FS_H
#define _VMS_FS_H

#include <sys/types.h>

/*
 * adapted by Chris Nott and Karl Maftoum and Amber East
 * from getvms.c by Carl Lydick (I think)
 */


#define VMS_BLOCKSIZE 512
#define VMS_BLOCKSIZE_BITS 9
#define VMS_MAXNAMLEN 20
#define VMS_SUPER_MAGIC 0x49554b44
#define VMS_ROOT_INO 4

struct vms_filename {
        char    f_nam[VMS_MAXNAMLEN];      /* File name (ASCII) */
        ushort  f_ver;          /* Version number */
};

struct vms_uic {
        ushort  u_prog;         /* Programmer number */
        ushort  u_proj;         /* Project number */
};

struct vms_fileid {
        ushort  f_num;          /* File number */
        ushort  f_seq;          /* File sequence number (worthless concept) */
        ushort  f_rvn;          /* Relative volume number (ditto and MBZ) */
};

struct vms_rms {
        char    f_forg;         /* Record format and file organization */
        char    f_ratt;         /* Record attributes */
        ushort  f_rsiz;         /* Record size */
        ushort  f_hvbn[2];      /* Highest VBN allocated */
        ushort  f_heof[2];      /* End of file block */
        ushort  f_ffby;         /* First free byte */
        char    f_bksz;         /* Bucket size */
        char    f_hdsz;         /* Fixed header size */
        ushort  f_mrs;          /* Maximum record size */
        ushort  f_deq;          /* Default extend quantity */
};

struct vms_ident {
        char    i_fnam[VMS_MAXNAMLEN];     /* File name */
        ushort  i_rvno;         /* Revision number */
        char    i_crdt[8];      /* Creation date and time */
        char    i_rvdt[8];      /* Revision date and time */
        char    i_exdt[8];      /* Expiration date and time */
        char    i_bkdt[8];      /* Backup date and time */
        char    i_ulab[80];     /* User label */
};

struct vms_header {
        char    h_idof;         /* Ident area offset */
        char    h_mpof;         /* Map area offset */
        char    h_acof;         /* Access control list offset */
        char    h_rsof;         /* Reserved area offset */
        ushort  h_fseg;         /* Extension segment number */
        ushort  h_flev;         /* Structure level and version */
        ushort  h_fnum;         /* File number */
        ushort  h_fseq;         /* File sequence number */
        ushort  h_frvn;         /* Relative volume number */
        ushort  h_efnu;         /* Extension file number */
        ushort  h_efsq;         /* Extension file sequence number */
        ushort  h_ervn;         /* Extension relative volume number */
        union {
                char    hu_ufat[32];    /* User file attributes */
                struct vms_rms hu_rms;      /* RMS file attributes */
        } h_ufat;
#define h_rms   h_ufat.hu_rms
        char    h_fcha[4];      /* File characteristics */
#define h_ucha  h_fcha[0]       /* User controlled characteristics */
#define h_scha  h_fcha[1]       /* System controlled characteristics */
        char    h_UU1[2];       /* Unused 1 */
        char    h_use;          /* Map words in use */
        char    h_priv;         /* Accessor privilege level */
        struct  vms_uic h_fown;      /* File owner UIC */
#define h_prog  h_fown.u_prog   /* Programmer (member) number */
#define h_proj  h_fown.u_proj   /* Project (group) number */
        ushort  h_fpro;         /* File protection code */
        ushort  h_rpro;         /* Record protection code */
	char	h_bfnu[6];	/* --> file header back link <-- */	 
        char    h_UU2[4];       /* Ununsed 2 */
/*        char    h_semk[4];      /* Security mask */
        struct  vms_ident h_ident;   /* Ident area */
        char    h_other[300];   /* Map area, access control area, etc */
};

struct vms_homeblock {
        long    hblb;         /* Home block LBN */
        long    ahlb;         /* Alternate home block LBN */
        long    ihlb;         /* Backup index file header LBN */
        char    vlev[2];      /* Structure level and version */
        ushort  sbcl;         /* Storage bitmap cluster factor */
        ushort  hbvb;         /* Home block VBN */
        ushort  ahvb;         /* Backup home block VBN */
        ushort  ihvb;         /* Backup index file header VBN */
        ushort  ibvb;         /* Index file bitmap VBN */
        ushort  iblb[2];      /* Index file bitmap LBN */
        long    fmax;         /* Maximum number of files */
        ushort  ibsz;         /* Index file bitmap size */
        ushort  rsvf;         /* Number of reserved files */
        ushort  dvty;         /* Disk device type */
        ushort  rvn;          /* Relative volume number */
        ushort  nvol;         /* Number of volumes */
        ushort  vcha;         /* Volume characteristics */
        struct  vms_uic vown;      /* Volume owner UIC */
        long    vsmx;         /* Volume security mask */
        ushort  vpro;         /* Volume protection code */
        ushort  dfpr;         /* Default file protection */
        ushort  drpr;         /* Default record protection */
        ushort  chk1;         /* First checksum */
        char    vdat[8];      /* Volume creation date */
        char    wisz;         /* Default window size */
        char    lruc;         /* Directory pre-access limit */
        ushort  fiex;         /* Default file extend */
        char    UU1[388];     /* Unused 1 */
        char    snam[12];     /* Structure name */
        char    indn[12];     /* Volume name */
        char    indo[12];     /* Volume owner */
        char    indf[12];     /* Format type */
        char    UU2[2];       /* Unused 2 */
        ushort  chk2;         /* Second checksum */
} hblock;

struct vms_directory {
        ushort  d_rbc;          /* Record byte count */
        ushort  d_vrlm;         /* Version limit */
        char    d_flags;        /* Flags */
        char    d_nbc;          /* Name byte count */
        char    d_fname[1];     /* File name string */
};

struct vms_dirval {
        ushort  d_ver;          /* Version number */
        struct  vms_fileid d_fid;    /* File ID */
};


#endif
